<?php

namespace App\Jobs;

use App\Models\EmailQueue;
use App\Models\EmailService;
use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Queue\SerializesModels;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Facades\Config;

class ProcessEmailQueue implements ShouldQueue
{
    use Dispatchable, InteractsWithQueue, Queueable, SerializesModels;

    public function __construct()
    {
        $this->onQueue('emails'); // Define a queue 'emails'
    }

    public function handle()
    {
        EmailService::where('status', 'active')->each(function ($service) {
            $pendingEmails = EmailQueue::where('email_service_id', $service->id)
                ->where('status', 'pending')
                ->take($service->batch_size)
                ->get();

            foreach ($pendingEmails as $email) {
                try {
                    // Configurar SMTP dinamicamente
                    Config::set('mail.mailers.smtp.host', $service->smtp_host);
                    Config::set('mail.mailers.smtp.port', $service->smtp_port);
                    Config::set('mail.mailers.smtp.username', $service->smtp_username);
                    Config::set('mail.mailers.smtp.password', $service->smtp_password);
                    Config::set('mail.mailers.smtp.encryption', $service->smtp_encryption);

                    
                    Mail::raw($email->body, function ($message) use ($email) {
                        $message->to($email->to)
                            ->subject($email->subject);
                    });
                    echo("\n:: ProcessEmailQueue\n");
                    

                    // Simula um pequeno delay
                    //sleep(1);

                    $email->update([
                        'status' => 'sent',
                        'sent_at' => now(),
                        'attempts' => $email->attempts + 1
                    ]);

                } catch (\Exception $e) {
                    $email->update([
                        'status' => 'failed',
                        'attempts' => $email->attempts + 1,
                        'last_attempt_at' => now()
                    ]);
                }
            }

            // Aguardar o intervalo configurado antes de processar o próximo lote
            sleep($service->batch_interval);
        });
    }
}
